﻿/*
 *         Progetto: Libreria di algoritmi di steganografia
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 14/01/2019
 *  
 *      Descrizione: Implementazione della classe SteganoStream.
 */

using System;
using System.Diagnostics.Contracts;
using System.IO;

namespace LibSteganografia
{
    [Serializable]
    [System.Runtime.InteropServices.ComVisible(true)]
    public enum SteganoStreamMode
    {
        Read = 0,
        Write = 1,
    }

    [System.Runtime.InteropServices.ComVisible(true)]
    public class SteganoStream : Stream, IDisposable
    {
        private String _path;
        private SteganoStreamMode _mode;
        MemoryStream _stream;
        BitmapHelper _bmp;

        public SteganoStream(String path, SteganoStreamMode mode)
        {
            FileAccess access = mode == SteganoStreamMode.Write ? FileAccess.Write : FileAccess.Read;
            _path = path;
            _mode = mode;
            _bmp = new BitmapHelper(_path, access);

            if (mode == SteganoStreamMode.Read)
                _stream = new MemoryStream(_bmp.getContent());
            else
                _stream = new MemoryStream();
        }
        public override bool CanRead
        {
            [Pure]
            get { return _mode == SteganoStreamMode.Read; }
        }

        public override bool CanSeek
        {
            [Pure]
            get { return false; }
        }

        public override bool CanWrite
        {
            [Pure]
            get { return _mode == SteganoStreamMode.Write; }
        }
        public override long Length => throw new NotImplementedException();

        public override long Position { get => throw new NotImplementedException(); set => throw new NotImplementedException(); }

        public override void Flush()
        {
            _stream.Flush();
        }

        public override int Read(byte[] buffer, int offset, int count)
        {
            return _stream.Read(buffer, offset, count);

        }

        public override long Seek(long offset, SeekOrigin origin)
        {
            throw new NotImplementedException();
        }

        public override void SetLength(long value)
        {
            throw new NotImplementedException();
        }

        public override void Write(byte[] buffer, int offset, int count)
        {
            _stream.Write(buffer, offset, count);
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                if (disposing)
                {
                    _bmp.putContent(_stream.ToArray());
                    _bmp.Dispose();
                }
            }
            finally
            {
                base.Dispose(disposing);
            }
        }
    }
}
