﻿/*
 *         Progetto: Libreria di algoritmi di steganografia
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 14/01/2019
 *  
 *      Descrizione: Test della classe SteganoStream.
 */

using LibSteganografia;
using System;
using System.Diagnostics;
using System.IO;
using System.Text;

namespace TestSteganoStream
{
    class Test
    {
        static void Main(string[] args)
        {
            TestStringa();      // Inserimento di una stringa 
            TestStringhe();     // Inserimento di un vettore di stringhe
            TestDocumento();    // Inserimento di un file
        }

        private static String SuffissoFile(String percorso, String suffisso)
        {
            var sb = new StringBuilder();

            return sb.ToString();
        }
        private static void TestDocumento()
        {
            const String nomeFileOrig = @".\paesaggio.orig.png";
            const string nomeDocOrig = @".\documento.orig.pdf";

            String nomeFile = nomeFileOrig.Replace(".orig", ".pdf");
            String nomeDoc = nomeDocOrig.Replace(".orig", "");
            File.Copy(nomeFileOrig, nomeFile, true);
            File.SetAttributes(nomeFile, File.GetAttributes(nomeFile) & ~FileAttributes.ReadOnly);

            if (File.Exists(nomeDoc))
                File.Delete(nomeDoc);

            Console.WriteLine();
            Console.WriteLine("Test documento...");
            try
            {
                Console.WriteLine("\tTEST DI INSERIMENTO");
                Console.WriteLine("\tImmagine iniziale: {0}.", nomeFileOrig);
                Console.WriteLine("\tDocumento inserito: {0}", nomeDocOrig);
                using (var ss = new SteganoStream(nomeFile, SteganoStreamMode.Write))
                using (var fs = new FileStream(nomeDocOrig, FileMode.Open))
                {
                    fs.CopyTo(ss);
                    Console.WriteLine("\tRisultato salvato in: {0}.", nomeFile);
                    Console.WriteLine();
                };

                using (var ss = new SteganoStream(nomeFile, SteganoStreamMode.Read))
                using (var fs = new FileStream(nomeDoc, FileMode.Create))
                {
                    Console.WriteLine("\tTEST DI ESTRAZIONE");
                    Console.WriteLine("\tImmagine iniziale: {0}.", nomeFile);
                    ss.CopyTo(fs);
                };

                Console.WriteLine("\tApertura del file estratto {0}", nomeDoc);
                Process.Start(nomeDoc);

            }
            catch (Exception ex)
            {
                Console.Error.WriteLine($"Errore: {ex.Message}");
                throw;
            }
        }

        private static void TestStringa()
        {
            const String nomeFileOrig = @".\albero.orig.png";
            const string s = "Test della libreria 'SteganoStream'";
            string result;

            String nomeFile = nomeFileOrig.Replace(".orig", ".str");
            File.Copy(nomeFileOrig, nomeFile, true);
            File.SetAttributes(nomeFile, File.GetAttributes(nomeFile) & ~FileAttributes.ReadOnly);

            Console.WriteLine("Test stringa...");
            try
            {
                Console.WriteLine("\tTEST DI INSERIMENTO");
                Console.WriteLine("\tImmagine iniziale: {0}.", nomeFileOrig);
                Console.WriteLine("\tStringa inserita: {0}", s);
                using (var ss = new SteganoStream(nomeFile, SteganoStreamMode.Write))
                using (var ms = new MemoryStream(Encoding.Unicode.GetBytes(s)))
                {
                    Debug.Assert(ss.CanRead == false && ss.CanWrite == true);
                    ms.CopyTo(ss);
                };
                Console.WriteLine("\tRisultato salvato in: {0}.", nomeFile);
                Console.WriteLine();

                using (var ss = new SteganoStream(nomeFile, SteganoStreamMode.Read))
                using (var ms = new MemoryStream())
                {
                    Console.WriteLine("\tTEST DI ESTRAZIONE");
                    Console.WriteLine("\tImmagine iniziale: {0}.", nomeFile);
                    Debug.Assert(ss.CanRead == true && ss.CanWrite == false);
                    ss.CopyTo(ms);
                    result = Encoding.Unicode.GetString(ms.ToArray());
                    Console.WriteLine("\tStringa estratta: {0}", result);
                };
            }
            catch (Exception ex)
            {
                Console.Error.WriteLine($"Errore: {ex.Message}");
                throw;
            }
        }

        static void TestStringhe()
        {
            const String nomeFileOrig = @".\albero.orig.png";
            const string s = "Questa è la riga di testo n. ";
            const int numStringhe = 5000;

            String nomeFile = nomeFileOrig.Replace(".orig", ".strmul");
            File.Copy(nomeFileOrig, nomeFile, true);
            File.SetAttributes(nomeFile, File.GetAttributes(nomeFile) & ~FileAttributes.ReadOnly);

            Console.WriteLine();
            Console.WriteLine("Test stringhe multiple... (inserimento di {0} stringhe)", numStringhe);
            try
            {
                using (var sw = new StreamWriter(new SteganoStream(nomeFile, SteganoStreamMode.Write)))
                {
                    Console.WriteLine("\tTEST DI INSERIMENTO");
                    Console.WriteLine("\tImmagine iniziale: {0}.", nomeFileOrig);
                    for (int i = 1; i <= numStringhe; i++)
                    {
                        string buffer = s + i.ToString();
                        sw.WriteLine(buffer);

                    }
                    Console.WriteLine("\tRisultato salvato in: {0}.", nomeFile);
                }

                using (var sr = new StreamReader(new SteganoStream(nomeFile, SteganoStreamMode.Read)))
                {
                    Console.WriteLine();
                    Console.WriteLine("\tTEST DI ESTRAZIONE");
                    Console.WriteLine("\tRighe estratte (campione):");
                    for (int i = 1; i <= numStringhe; i++)
                    {
                        string buffer = sr.ReadLine();
                        if ((i % (numStringhe / 10)) == 0) Console.WriteLine("\t{0}", buffer);
                    }
                }
            }
            catch (Exception ex)
            {

                Console.Error.WriteLine($"Errore: {ex.Message}");
                throw;
            }

        }
    }
}
